import winreg


def is_show_file_ext():
    """检查是否显示文件扩展名"""
    key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, 'Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Advanced')
    value, _ = winreg.QueryValueEx(key, 'HideFileExt')
    winreg.CloseKey(key)
    return value == 0


def set_show_file_ext():
    """设置显示文件扩展名"""
    key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, 'Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Advanced', 0, winreg.KEY_WRITE)
    winreg.SetValueEx(key, 'HideFileExt', 0, winreg.REG_DWORD, 0)
    winreg.CloseKey(key)







import xbot # type: ignore
import os
import json
import inspect


# 影刀应用运行时缓存数据
# 缓存目录 考虑跨平台
# %locapappdata% -> xbot -> app -> uuid
# %locapappdata% -> xbot -> activity -> activity_name


def get_xbot_type():
    """获取机器人类型
    :retrun: app, activity
    """
    stack = inspect.stack()
    for calling_module_frame in stack:
        if "xbot_robot" in calling_module_frame.filename:
            calling_module_file_path =  calling_module_frame.filename
            break
    calling_module_dir, _ = os.path.split(calling_module_file_path)
    package_json_file_path = os.path.join(calling_module_dir, "package.json")

    with open(package_json_file_path) as f:
        package_json = json.load(f)
        robot_type = package_json.get("robot_type")

        # 应用返回对应的 uuid , 指令返回指令编码
        if robot_type == "app":
            robot_key = package_json.get("uuid")

        if robot_type == "activity":
            robot_key = package_json.get("activity_code")

        return robot_type, robot_key


def get_xbot_cache_path():
    """获取缓存文件路径"""
    cache_path = os.environ.get('LOCALAPPDATA')
    xbot_type = get_xbot_type()
    xbot_cache_path = os.path.join(cache_path, "xbot", *xbot_type)

    if not os.path.exists(xbot_cache_path):
        os.makedirs(xbot_cache_path)

    default_cache_path = os.path.join(xbot_cache_path, "default")
    if not os.path.exists(default_cache_path):
        with open(default_cache_path, "w") as f:
            json.dump(dict(), f)

    return xbot_cache_path


def get_xbot_cache_default(key=None):
    """返回应用或者指令缓存数据
    :param: key 获取键, 如果是 None 返回全部
    """
    default_cache_path = os.path.join(get_xbot_cache_path(), "default")

    with open(default_cache_path) as f:
        default_json = json.load(f)
        if key:
            return default_json.get(key)
        return default_json


def set_xbot_cache_default(key, value) -> bool:
    """设置应用或指令缓存数据"""

    try:
        default_cache_path = os.path.join(get_xbot_cache_path(), "default")
        with open(default_cache_path, "r") as fr:
            default_json = json.load(fr)
        with open(default_cache_path, "w") as fw:
            default_json[key] = value
            json.dump(default_json, fw, indent=4)
        return True
    except Exception as e:
        print(e)
        return False


def set_xbot_cache_default_breakponit(num: int):
    """设置应用执行过程的断点"""
    set_xbot_cache_default("breakpoint", num)


def get_xbot_cache_default_breakponit() -> int:
    """获取应用执行过程中断点"""
    return get_xbot_cache_default("breakpoint")


def show_xbot_cache_default_breakpoint(title="继续运行", label="开始位置") -> None:
    """通过对话框的形式展示断点, 并允许客户输入"""
    breakponit_value = get_xbot_cache_default_breakponit()
    if get_xbot_cache_default_breakponit() == None:
        return

    dialog_info = xbot.app.dialog.show_input_dialog(title, label, typestr='input', value=str(breakponit_value))
    breakponit_value = dialog_info.get('value')
    assert str(breakponit_value).isdigit(), "请输入数字"
    set_xbot_cache_default_breakponit(int(breakponit_value))




# 获取客户端的版本, 安装路径
# 获取当前登录的用户名
import os
import winreg
import xml.dom.minidom
import win32com.client
import win32api
import win32con
import win32process

__all__ = ["get_current_version", "get_current_user"]

def read_registry_value(key, subkey, value_name):
    try:
        registry_key = winreg.OpenKey(key, subkey, 0, winreg.KEY_READ)
        value, _ = winreg.QueryValueEx(registry_key, value_name)
        winreg.CloseKey(registry_key)
        return value
    except FileNotFoundError:
        return None


def write_registry_value(key, subkey, value_name, value):
    try:
        registry_key = winreg.OpenKey(key, subkey, 0, winreg.KEY_WRITE)
        winreg.SetValueEx(registry_key, value_name, 0, winreg.REG_SZ, value)
        winreg.CloseKey(registry_key)
        return True
    except Exception as e:
        print("Error writing to registry:", e)
        return False


def get_process_path(process_name):
    wmi = win32com.client.GetObject("winmgmts:")
    for process in wmi.InstancesOf("Win32_Process"):
        if process.Name == process_name:
            pid = process.ProcessId
            try:
                process_handle = win32api.OpenProcess(
                    win32con.PROCESS_QUERY_INFORMATION, False, pid
                )
                exe_path = win32process.GetModuleFileNameEx(process_handle, 0)
                return exe_path
            except Exception as e:
                print(f"Failed to get process path: {e}")
                return None
            finally:
                win32api.CloseHandle(process_handle)
    return None


def get_current_version():
    """获取当前影刀版本"""
    process_name = "ShadowBot.Shell.exe"
    process_path = get_process_path(process_name)
    try:
        # 获取文件版本信息
        info = win32api.GetFileVersionInfo(process_path, "\\")
        version = "%d.%d.%d.%d" % (
            info["FileVersionMS"] / 65536,
            info["FileVersionMS"] % 65536,
            info["FileVersionLS"] / 65536,
            info["FileVersionLS"] % 65536,
        )
        return version
    except Exception as e:
        return None


def get_current_user():
    """获取当前影刀登录账号"""
    path_username = rf'{os.getenv("Localappdata")}\ShadowBot\users\Account.xml'
    path_dom = xml.dom.minidom.parse(path_username)
    usernam_data = (
        path_dom.documentElement.getElementsByTagName("UserName")[0].childNodes[0].data
    )
    return usernam_data



# 获取 %localappdata% 文件夹的数据

import os
import json

local_appdata_path = os.getenv('LOCALAPPDATA')


def get_chromium_user_data_path(browser_name="edge") -> str:
    """获取浏览器的用户数据目录"""
    if browser_name == "edge":
        return os.path.join(local_appdata_path, "Microsoft", "Edge", "User Data")
    elif browser_name == "chrome":
        return os.path.join(local_appdata_path, "Google", "Chrome", "User Data")
    else:
        raise ValueError("不支持的浏览器名称")








from xbot import web


def get_web_page_by_web_page(web_page):
    """根据网页对象获取网页对象"""
    product_name = getattr(
        web_page, "product_name",
        "cef" if web_page._controller == "CEFBrowser" else "firefox")
    return web.get_active(mode=product_name)


def get_web_page_product_name(web_page):
    """根据网页对象获取浏览器的名称"""
    product_name = getattr(
        web_page, "product_name",
        "cef" if web_page._controller == "CEFBrowser" else "firefox")
    return product_name
